#include "rational.hpp"
#include <sstream>
#include <iostream>

/*
	http://cppbook.com/index.php?title=Io_ios_setstate
*/

Rational::Rational(int z, int n) : zaehler(z), nenner(n) {
	if(nenner < 0){
		nenner = -nenner;
		zaehler = -zaehler;
	}
}

std::string Rational::toString() const {
	std::stringstream strstr;
	strstr << zaehler << "/" << nenner;
	return strstr.str();
}

Rational Rational::operator*(const Rational& rechts) const {
	Rational ergebnis(this->zaehler * rechts.zaehler, this->nenner * rechts.nenner);
	return ergebnis;
}

Rational& Rational::operator*=(const Rational& rechts) {
	this->zaehler *= rechts.zaehler;
	this->nenner *= rechts.nenner;
	return *this;
}

Rational Rational::operator+(const Rational& rechts) const {
	Rational ergebnis(this->zaehler*rechts.nenner + rechts.zaehler*this->nenner, this->nenner*rechts.nenner);
	return ergebnis;
}

Rational& Rational::operator+=(const Rational& rechts) {
	this->zaehler = this->zaehler*rechts.nenner + rechts.zaehler*this->nenner;
	this->nenner *= rechts.nenner;
	return *this;
}

Rational Rational::operator-(const Rational& rechts) const {
	Rational ergebnis(this->zaehler*rechts.nenner - rechts.zaehler*this->nenner, this->nenner*rechts.nenner);
	return ergebnis;
}

Rational& Rational::operator-=(const Rational& rechts) {
	this->zaehler = this->zaehler*rechts.nenner - rechts.zaehler*this->nenner;
	this->nenner *= rechts.nenner;
	return *this;
}

Rational Rational::operator/(const Rational& rechts) const {
	Rational ergebnis(this->zaehler / rechts.zaehler, this->nenner / rechts.nenner);
	return ergebnis;
}

Rational& Rational::operator/=(const Rational& rechts) {
	this->zaehler /= rechts.zaehler;
	this->nenner /= rechts.nenner;
	return *this;
}

bool Rational::operator==(const Rational& rechts) {
	return this->zaehler*rechts.nenner == rechts.zaehler*this->nenner;
}

bool Rational::operator!=(const Rational& rechts) {
	return this->zaehler*rechts.nenner != rechts.zaehler*this->nenner;
}

bool Rational::operator<(const Rational& rechts) {
	return this->zaehler*rechts.nenner < rechts.zaehler*this->nenner;
}

bool Rational::operator<=(const Rational& rechts) {
	return this->zaehler*rechts.nenner <= rechts.zaehler*this->nenner;
}

bool Rational::operator>(const Rational& rechts) {
	return this->zaehler*rechts.nenner > rechts.zaehler*this->nenner;
}

bool Rational::operator>=(const Rational& rechts) {
	return this->zaehler*rechts.nenner >= rechts.zaehler*this->nenner;
}

std::ostream& operator<<(std::ostream& os, const Rational& r){
	os << r.toString();
	return os;
}

std::istream& operator>>(std::istream& is, Rational& r){
	//input stream in string packen
	std::string str;
	is >> str;
	//variable für position des bruchstrichs
	int pos;
	//wenn kein bruchstrich gefunden wurde, failbit setzen und fehlermeldung ausgeben
	if((pos = str.find("/")) == std::string::npos){
		is.setstate(std::ios::failbit);
		std::cout << "inkorrekte Syntax der Eingabe" << std::endl;
		return is;
	}
	//string bis zum bruchstrich in int wandeln und in zaehler packen
	std::stringstream strstr1, strstr2;
	strstr1 << str.substr(0, pos);
	strstr1 >> r.zaehler;
	//string nach dem bruchstrich in int wandeln und in nenner packen
	strstr2 << str.substr(pos+1, str.length()-1);
	strstr2 >> r.nenner;
	return is;
}

